/*******************************************************************************
 * Copyright (c) 2000, 2019 IBM Corp. and others
 *
 * This program and the accompanying materials are made available under
 * the terms of the Eclipse Public License 2.0 which accompanies this
 * distribution and is available at https://www.eclipse.org/legal/epl-2.0/
 * or the Apache License, Version 2.0 which accompanies this distribution and
 * is available at https://www.apache.org/licenses/LICENSE-2.0.
 *
 * This Source Code may also be made available under the following
 * Secondary Licenses when the conditions for such availability set
 * forth in the Eclipse Public License, v. 2.0 are satisfied: GNU
 * General Public License, version 2 with the GNU Classpath
 * Exception [1] and GNU General Public License, version 2 with the
 * OpenJDK Assembly Exception [2].
 *
 * [1] https://www.gnu.org/software/classpath/license.html
 * [2] http://openjdk.java.net/legal/assembly-exception.html
 *
 * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
 *******************************************************************************/

#ifndef PROFILEGENERATOR_INCL
#define PROFILEGENERATOR_INCL

#include <stdint.h>
#include "optimizer/Optimization.hpp"
#include "optimizer/OptimizationManager.hpp"

namespace TR { class CFG; }
namespace TR { class Node; }
namespace TR { class TreeTop; }

// Profile generator
//
// Create a sample-based profiling version of the method
//

class TR_ProfileGenerator : public TR::Optimization
   {
   public:

   TR_ProfileGenerator(TR::OptimizationManager *manager);
   static TR::Optimization *create(TR::OptimizationManager *manager)
      {
      return new (manager->allocator()) TR_ProfileGenerator(manager);
      }

   virtual int32_t perform();
   virtual const char * optDetailString() const throw();

   private :

   int32_t    prepareBlocks();
   void       createProfiledMethod();
   TR::Node    *copyRegDeps(TR::Node *from, bool shareChildren);

   TR::CFG     *_cfg;
   TR::TreeTop *_firstOriginalTree;
   TR::TreeTop *_lastOriginalTree;
   static bool      _dynamnicProfilingCount;
   TR::TreeTop *_asyncTree;
   };

#endif
